from django.contrib.auth.hashers import check_password
from usermerge.models import Admin, Platform, Registry, User

# Create your backends here.
# https://docs.djangoproject.com/en/2.0/topics/auth/customizing/#other-authentication-sources

# For more information on Admin, Platform, Registry and User models, see models.py .
# For more information on the platform names that are provided in the drop-down list of the login form, see login.html and get_names_of_SoftLab_provided_platforms_from_DB() function of views.py .
# For more information on password management functions (e.g. check_password()), see https://docs.djangoproject.com/en/2.0/topics/auth/passwords/ .

class UserBackend:
    def authenticate(self, request, platform, username, password):
        """
        Authenticate the credentials POSTed via the login form.

        If the selected platform is SLUB, the user should be authenticated as Admin with the happy path being:
        * Try to find an Admin instance of usermergeDB that corresponds to the provided username.
        * If a corresponding instance is found, check if the provided password matches the one of the instance.
        * If the provided password matches the one of the instance, the user is authenticated successfully as Admin and
          the instance is returned.

        If the selected platform is not SLUB, the user should be authenticated as User with the happy path being:
        * Try to find a Registry instance of usermergeDB that corresponds to the provided platform and username.
        * If a corresponding instance is found, check if the provided password matches the one of the instance.
        * If the provided password matches the one of the instance, the user is authenticated successfully as User and
          the User instance that is associated with the corresponding Registry instance is returned.

        If any of the checks (finding the corresponding instance and comparing the passwords) fails, the authentication
        fails and an empty instance (None) is returned (if the first check fails, the second is never performed).
        """
        if platform != 'SLUB':
            # User authentication
            platform_instance = Platform.objects.get(name = platform)
            try:
                registry = Registry.objects.get(platform = platform_instance, username = username)
            except Registry.DoesNotExist:
                return None
            else:
                if check_password(password, registry.password):
                    return registry.user
                else:
                    return None
        else:
            # Admin authentication
            try:
                admin = Admin.objects.get(username = username)
            except Admin.DoesNotExist:
                return None
            else:
                if check_password(password, admin.password):
                    return admin
                else:
                    return None

    def get_user(self, user_class, user_id):
        """
        Return the user instance associated with the given class/model (User/Admin) and id.
        If no associated user instance is found, return an empty instance (None).
        """
        if user_class == 'usermerge.User':
            try:
                return User.objects.get(pk = user_id)
            except User.DoesNotExist:
                return None
        else:
            try:
                return Admin.objects.get(pk = user_id)
            except Admin.DoesNotExist:
                return None

