import re
from django import forms
from usermerge.models import Platform

# Create your forms here.
# https://docs.djangoproject.com/en/2.0/ref/forms/

# The below-defined forms are based on the models defined in models.py .
# For more information on clean_<fieldname>() and clean() methods, see https://docs.djangoproject.com/en/2.0/ref/forms/validation/ .
# For more information on the platform names that are provided in the drop-down list of the login form, see login.html and get_names_of_SoftLab_provided_platforms_from_DB() function of views.py .

class LoginForm(forms.Form):
    platform = forms.CharField(error_messages = {'required': 'Το δηλωθέν όνομα πλατφόρμας δεν ανταποκρίνεται σε καμία από τις '
                                                             'υποστηριζόμενες πλατφόρμες της βάσης!'})
    username = forms.CharField(max_length = 50,
                               error_messages = {'required': 'Το όνομα χρήστη πρέπει απαραίτητα να συμπληρωθεί!',
                                                 'max_length': 'Το όνομα χρήστη δεν πρέπει να υπερβαίνει τους 50 χαρακτήρες!'})
    password = forms.CharField(max_length = 50,
                               error_messages = {'required': 'Ο κωδικός πρόσβασης πρέπει απαραίτητα να συμπληρωθεί!',
                                                 'max_length': 'Ο κωδικός πρόσβασης δεν πρέπει να υπερβαίνει τους 50 χαρακτήρες!'})

    def clean_platform(self):
        platform = self.cleaned_data['platform']
        # Ensure that the platform (name) is selected among the provided ones in the drop-down list of the login form.
        # If it is not (e.g. it is misedited via JavaScript), raise a ValidationError that will be printed under the form.
        if platform == 'SLUB' or (platform != 'ECE-NTUA' and platform in Platform.objects.values_list('name', flat = True)):
            return platform
        else:
            raise forms.ValidationError('Το δηλωθέν όνομα πλατφόρμας δεν ανταποκρίνεται σε καμία '
                                        'από τις υποστηριζόμενες πλατφόρμες της βάσης!')

    def clean(self):
        cleaned_data = super().clean()
        # If platform and username have survived the initial individual field checks (by the time the form’s clean() method is called,
        # all the individual field clean methods will have been run, so cleaned_data will be populated with any data that has survived
        # so far) and platform refers either to Novice or to Grader, ensure that the username format is piYYbRRR (if it is not, raise a
        # ValidationError that will be printed under the login form), where YY refers to year and RRR refers to username serial number.
        if set(('platform', 'username')).issubset(cleaned_data):
            platform = cleaned_data['platform']
            username = cleaned_data['username']
            if platform == 'Novice' or platform == 'Grader':
                if not re.match('pi[0-9]{2}b[0-9]{3}', username):
                    raise forms.ValidationError('Το όνομα χρήστη στις πλατφόρμες Novice και Grader πρέπει να είναι '
                                                'της μορφής piYYbRRR (ΥΥ: έτος, RRR: σειριακός αριθμός ονόματος)!')

