from django.db import models
from django.utils.crypto import salted_hmac

# Create your models here.
# https://docs.djangoproject.com/en/2.0/ref/models/

# Regarding the fields whose null attribute/option is set to True (WARNING: Any unique fields whose null attribute/option is undeclared, i.e. False by default, should NEVER be allowed to be empty): The empty DB value for DateTimeFields is NULL (interpreted as None in Python), thus designating the absence of value. The empty DB value for CharFields and EmailFields is the empty/zero-length string (''), which is a valid string value. For unique CharFields and EmailFields, such as ece_id and email of the User model, whose value may NOT always be declared when inserting/updating the respective DB entries/records, this could lead in integrity errors ("Duplicate entry '' for key 'ece_id'/'email'"). To avoid such errors, NULL should be used as the empty DB value of these fields (the empty string should NOT be used in this case). For more information, see https://docs.djangoproject.com/en/2.0/ref/models/fields/ .
# first_name and last_name fields can contain greek characters. The length of each greek character is 2 bytes, while the length of each english/punctuation character is 1 byte.
# Django creates hashed passwords using the PBKDF2 algorithm with a SHA256 hash by default (https://docs.djangoproject.com/en/2.0/topics/auth/passwords/). The length of each hashed password is 78 bytes.
# The ECE-ID format is 031YYRRR (YY: year, RRR: registration number).
# The user_logged_in signal (see login() function in auth.py) triggers the update of the last_login fields, so the latter are necessary.
# Original source code for get_session_auth_hash() methods and is_authenticated attributes can be found in https://github.com/django/django/blob/master/django/contrib/auth/base_user.py (for more information, see https://docs.djangoproject.com/en/2.0/topics/auth/customizing/). get_session_auth_hash() methods are needed in login() and get_user() functions of auth.py, while is_authenticated attributes are needed for login_required() decorator of views.py .

class User(models.Model):
    first_name = models.CharField(max_length = 100)
    last_name = models.CharField(max_length = 100)
    ece_id = models.CharField(max_length = 8, unique = True, null = True)
    email = models.EmailField(unique = True, null = True)
    last_login = models.DateTimeField(null = True)

    def get_session_auth_hash(self):
        """
        Return an HMAC of the email field.
        """
        key_salt = 'usermerge.models.User.get_session_auth_hash'
        return salted_hmac(key_salt, self.email).hexdigest()

    @property
    def is_authenticated(self):
        """
        Always return True. This is a way to tell if the user has been
        authenticated in views.
        """
        return True

class Platform(models.Model):
    name = models.CharField(max_length = 50, unique = True)

class Registry(models.Model):
    user = models.ForeignKey('User', on_delete = models.PROTECT)
    platform = models.ForeignKey('Platform', on_delete = models.PROTECT)
    username = models.CharField(max_length = 50)
    password = models.CharField(max_length = 100)

    class Meta:
        unique_together = (('user', 'platform',), ('platform', 'username',),)

class Admin(models.Model):
    first_name = models.CharField(max_length = 100)
    last_name = models.CharField(max_length = 100)
    email = models.EmailField(unique = True)
    username = models.CharField(max_length = 50, unique = True)
    password = models.CharField(max_length = 100)
    last_login = models.DateTimeField(null = True)

    def get_session_auth_hash(self):
        """
        Return an HMAC of the email field.
        """
        key_salt = 'usermerge.models.Admin.get_session_auth_hash'
        return salted_hmac(key_salt, self.email).hexdigest()

    @property
    def is_authenticated(self):
        """
        Always return True. This is a way to tell if the user has been
        authenticated in views.
        """
        return True

